from pylab import *

from pyana.cena import gfactor

gf = gfactor.GFactorH_Component()
g_ch3 = gf.geometricFactor()[:, 3]

from pyana.cena import energy
energy_in_eV = energy.getEnergyE16()
ge_i = g_ch3 * energy_in_eV

import math
from pyana.util import maxwell
import numpy as np
import scipy
delta_e = energy.getDeltaE16()
bound_e = np.array(energy.getRangeE16())
Mij = np.zeros([16, 16])

figure(figsize=(12,12))
subplot(221)

for i in range(16):
    ei = energy_in_eV[i]

### Assuming Maxwell
    def shape_maxwell(x):
        sigma = ei / math.sqrt(-8 * math.log(0.5))
        mx = maxwell.mk1d(1, ei, sigma)
        return mx(x)

## Assuming triangle
    def shape_triangle(x):
        if x < 0.2 * ei:
            return 0
        elif x < ei:
            return ((x - ei) / (0.8) + ei) / ei
        elif x < 2.2 * ei:
            return ( ei - (x - ei) / (1.2)) / ei
        else:
            return 0

### Assuming narrow Maxwell
    def shape_narrow_maxwell(x):
        sigma = 10
        mx = maxwell.mk1d(1, ei, sigma)
        return mx(x)

### Assuming box distribution
    def shape_box(x):
        if x < bound_e[0, i] or bound_e[1, i] < x:
            return 0.0
        else:
            return 1.0

#    shape_energy_response = shape_maxwell
    shape_energy_response = shape_triangle
#    shape_energy_response = shape_narrow_maxwell
#    shape_energy_response = shape_box
            
    subplot(221)
    xdetail = arange(5000)
    ydetail = [shape_energy_response(x) for x in xdetail]
    plot(xdetail, ydetail, '-x', label='E-%d' % i)
#    xscale('log')
    yscale('linear')
    ylim(0, 1)
    xlabel('Energy [eV]')
    ylabel('Response')
    title('Assumed relative energy response')
    gi_j = np.array([shape_energy_response(ej) for ej in energy_in_eV])
#    gi_j = np.array([scipy.integrate.quad(shape_energy_response, bound_e[0, ie], bound_e[1, ie])[0]/delta_e[ie] for ie in range(16)])
    print gi_j
    gi_j_sum = (gi_j * delta_e).sum()
    gi_j *= ge_i[i] / gi_j_sum
    Mij[i, :] = gi_j * delta_e

    subplot(222)
    plot(energy_in_eV, Mij[i, :], '-+', label='E-%d' % i)

subplot(222)
plot(energy_in_eV, ge_i, '*')
title('Differential g_factor + total g_factor (Star)')
xscale('log')
yscale('log')
ylim(1e-10, 1)
xlabel('Energy [eV]')
ylabel('Mij (i fixed)')

iarr = arange(17)
jarr = arange(17)
ii, jj = meshgrid(iarr, jarr)
#subplot(222)
#from matplotlib.colors import LogNorm
#pcolor(ii, jj, Mij.T+1e-25, norm=LogNorm(vmin=1e-10, vmax=Mij.max()))
#colorbar()

print 'ge_i = ', ge_i
print 'Mij = ', Mij

Minv = np.linalg.inv(Mij)
Minvxdt = Minv / .375
print Minvxdt
Mij.dot(Minv)
print Mij.dot(Minv)

from pylab import *
import pyana.util.bipower as bipower
Jtheory = bipower.mkfunc(1.6e4, -0.15, 1.4e12, -3.25)
J = [Jtheory(e) for e in energy_in_eV]
subplot(223)
plot(energy_in_eV, J, label='Assumed')
xscale('log')
yscale('log')
title('Energy spectra')
xlabel('Energy [eV]')
ylabel('Flux [#/cm2 sr eV s]')
legend()
J = np.array(J)
print 'J(theory)=', J
Cexp = Mij.dot(J) * .375
Cexp *= 100
print 'Cexp=', Cexp

subplot(224)
plot(energy_in_eV, Cexp, '-+', label='Expected: G-factor matrix')
xscale('log')
yscale('linear')
title('Count rate')
ylabel('Cnt * 100')

# For ideal sensor
Cexp2 = J * ge_i * 0.375
plot(energy_in_eV, Cexp2 * 100, '--*', label='Expected: G-factor')

# Random noise
np.random.seed(10000)
Cnoise = np.random.randn(16) * 3
Cobs = np.round(Cexp + Cnoise)

for i in range(16):
    if Cobs[i] < 0:
        Cobs[i] = 0

plot(energy_in_eV, Cobs, '-o', label='Sample data: noise + round-off')

legend()

ylim(0, 100)

subplot(223)
# Nominal way
Jrevnom = Cobs / ge_i / 0.375 / 100
plot(energy_in_eV, np.ma.masked_less_equal(Jrevnom, 0), '-v', label='Derived fr. Sample: G-factor')

# Inversion. Nominal
Jrev = Minv.dot(Cobs) / 0.375 / 100   # 100 is to revert the offset.
plot(energy_in_eV, np.ma.masked_less_equal(Jrev, 0), '-x', label='Derived fr. Sample: Inverse matrix')

# Inversion, NNLS
from scipy.optimize import nnls
Jrevnnls = nnls(Mij, Cobs)[0] / 0.375 / 100
plot(energy_in_eV, np.ma.masked_less_equal(Jrevnnls, 0), '-o', label='Derived fr. Sample: NNLS')

ylim(1, 1e8)

legend()

savefig('topic_cena_energy_gfactor_matrix.png')
